@@ -2,7 +2,6 @@
 from typing import Dict
 from lightwood.helpers.templating import call, inline_dict, align
 from lightwood.api import dtype
-import numpy as np
 from lightwood.api.types import (
     JsonAI,
     TypeInformation,
@@ -17,7 +16,7 @@ from lightwood.helpers.log import log
 
 # For custom modules, we create a module loader with necessary imports below
 IMPORT_EXTERNAL_DIRS = """
-for import_dir in [os.path.expanduser('~/lightwood_modules'), '/etc/lightwood_modules']:
+for import_dir in [os.path.join(os.path.expanduser('~/lightwood_modules'), lightwood_version.replace('.', '_')), os.path.join('/etc/lightwood_modules', lightwood_version.replace('.', '_'))]:
     if os.path.exists(import_dir) and os.access(import_dir, os.R_OK):
         for file_name in list(os.walk(import_dir))[0][2]:
             if file_name[-3:] != '.py':
@@ -29,10 +28,11 @@ for import_dir in [os.path.expanduser('~/lightwood_modules'), '/etc/lightwood_mo
             loader.exec_module(module)
             sys.modules[mod_name] = module
             exec(f'import {mod_name}')
-"""
+""" # noqa
 
 IMPORTS = """
 import lightwood
+from lightwood import __version__ as lightwood_version
 from lightwood.analysis import *
 from lightwood.api import *
 from lightwood.data import *
@@ -53,6 +53,7 @@ import os
 from types import ModuleType
 import importlib.machinery
 import sys
+import time
 """
 
 
@@ -80,29 +81,29 @@ def lookup_encoder(
 
     tss = problem_defintion.timeseries_settings
     encoder_lookup = {
-        dtype.integer: "Integer.NumericEncoder",
-        dtype.float: "Float.NumericEncoder",
-        dtype.binary: "Binary.BinaryEncoder",
-        dtype.categorical: "Categorical.CategoricalAutoEncoder"
+        dtype.integer: "NumericEncoder",
+        dtype.float: "NumericEncoder",
+        dtype.binary: "BinaryEncoder",
+        dtype.categorical: "CategoricalAutoEncoder"
         if statistical_analysis is None
         or len(statistical_analysis.histograms[col_name]) > 100
-        else "Categorical.OneHotEncoder",
-        dtype.tags: "Tags.MultiHotEncoder",
-        dtype.date: "Date.DatetimeEncoder",
-        dtype.datetime: "Datetime.DatetimeEncoder",
-        dtype.image: "Image.Img2VecEncoder",
-        dtype.rich_text: "Rich_Text.PretrainedLangEncoder",
-        dtype.short_text: "Short_Text.CategoricalAutoEncoder",
-        dtype.array: "Array.ArrayEncoder",
-        dtype.tsarray: "TimeSeries.TimeSeriesEncoder",
-        dtype.quantity: "Quantity.NumericEncoder",
-        dtype.audio: "Audio.MFCCEncoder"
+        else "OneHotEncoder",
+        dtype.tags: "MultiHotEncoder",
+        dtype.date: "DatetimeEncoder",
+        dtype.datetime: "DatetimeEncoder",
+        dtype.image: "Img2VecEncoder",
+        dtype.rich_text: "PretrainedLangEncoder",
+        dtype.short_text: "CategoricalAutoEncoder",
+        dtype.array: "ArrayEncoder",
+        dtype.tsarray: "TimeSeriesEncoder",
+        dtype.quantity: "NumericEncoder",
+        dtype.audio: "MFCCEncoder"
     }
 
     # If column is a target, only specific feature representations are allowed that enable supervised tasks
     target_encoder_lookup_override = {
-        dtype.rich_text: "Rich_Text.VocabularyEncoder",
-        dtype.categorical: "Categorical.OneHotEncoder",
+        dtype.rich_text: "VocabularyEncoder",
+        dtype.categorical: "OneHotEncoder",
     }
 
     # Assign a default encoder to each column.
@@ -118,8 +119,12 @@ def lookup_encoder(
         if col_dtype in (dtype.categorical, dtype.binary):
             if problem_defintion.unbias_target:
                 encoder_dict["args"][
-                    "target_class_distribution"
-                ] = "$statistical_analysis.target_class_distribution"
+                    "target_weights"
+                ] = "$statistical_analysis.target_weights"
+            if problem_defintion.target_weights is not None:
+                encoder_dict["args"][
+                    "target_weights"
+                ] = problem_defintion.target_weights
 
         if col_dtype in (dtype.integer, dtype.float, dtype.array, dtype.tsarray):
             encoder_dict["args"][
@@ -130,7 +135,7 @@ def lookup_encoder(
     if tss.is_timeseries:
         gby = tss.group_by if tss.group_by is not None else []
         if col_name in tss.order_by + tss.historical_columns:
-            encoder_dict["module"] = col_dtype.capitalize() + ".TimeSeriesEncoder"
+            encoder_dict["module"] = "TimeSeriesEncoder"
             encoder_dict["args"]["original_type"] = f'"{col_dtype}"'
             encoder_dict["args"]["target"] = "self.target"
             encoder_dict["args"]["grouped_by"] = f"{gby}"
@@ -138,24 +143,24 @@ def lookup_encoder(
         if is_target:
             if col_dtype in [dtype.integer]:
                 encoder_dict["args"]["grouped_by"] = f"{gby}"
-                encoder_dict["module"] = "Integer.TsNumericEncoder"
+                encoder_dict["module"] = "TsNumericEncoder"
             if col_dtype in [dtype.float]:
                 encoder_dict["args"]["grouped_by"] = f"{gby}"
-                encoder_dict["module"] = "Float.TsNumericEncoder"
+                encoder_dict["module"] = "TsNumericEncoder"
             if tss.nr_predictions > 1:
                 encoder_dict["args"]["grouped_by"] = f"{gby}"
                 encoder_dict["args"]["timesteps"] = f"{tss.nr_predictions}"
-                encoder_dict["module"] = "TimeSeries.TsArrayNumericEncoder"
+                encoder_dict["module"] = "TsArrayNumericEncoder"
         if "__mdb_ts_previous" in col_name:
-            encoder_dict["module"] = "Array.ArrayEncoder"
+            encoder_dict["module"] = "ArrayEncoder"
             encoder_dict["args"]["original_type"] = f'"{tss.target_type}"'
             encoder_dict["args"]["window"] = f"{tss.window}"
 
     # Set arguments for the encoder
-    if encoder_dict["module"] == "Rich_Text.PretrainedLangEncoder" and not is_target:
+    if encoder_dict["module"] == "PretrainedLangEncoder" and not is_target:
         encoder_dict["args"]["output_type"] = "$dtype_dict[$target]"
 
-    if eval(encoder_dict["module"].split(".")[1]).is_trainable_encoder:
+    if eval(encoder_dict["module"]).is_trainable_encoder:
         encoder_dict["args"]["stop_after"] = "$problem_definition.seconds_per_encoder"
 
     if is_target_predicting_encoder:
@@ -181,15 +186,18 @@ def generate_json_ai(
     exec(IMPORT_EXTERNAL_DIRS, globals())
     target = problem_definition.target
     input_cols = []
+    tss = problem_definition.timeseries_settings
+
     for col_name, col_dtype in type_information.dtypes.items():
         if (
-            col_name not in type_information.identifiers
-            and col_dtype not in (dtype.invalid, dtype.empty)
-            and col_name != target
+                (col_name not in type_information.identifiers
+                 and col_dtype not in (dtype.invalid, dtype.empty)
+                 and col_name != target)
+                or
+                (tss.group_by is not None and col_name in tss.group_by)
         ):
             input_cols.append(col_name)
 
-    tss = problem_definition.timeseries_settings
     is_target_predicting_encoder = False
     is_ts = problem_definition.timeseries_settings.is_timeseries
     # Single text column classification
@@ -310,7 +318,7 @@ def generate_json_ai(
 
         if (
             tss.is_timeseries
-            and eval(encoder["module"].split(".")[1]).is_timeseries_encoder
+            and eval(encoder["module"]).is_timeseries_encoder
         ):
             if tss.group_by is not None:
                 for group in tss.group_by:
@@ -351,56 +359,30 @@ def generate_json_ai(
         if list(outputs.values())[0].data_dtype in [dtype.integer, dtype.float]:
             accuracy_functions = ["evaluate_array_accuracy"]
 
-    if problem_definition.time_aim is None and (
-        problem_definition.seconds_per_mixer is None
-        or problem_definition.seconds_per_encoder is None
-    ):
-        problem_definition.time_aim = (
-            1000
-            + np.log(statistical_analysis.nr_rows / 10 + 1)
-            * np.sum(
-                [
-                    4
-                    if x
-                    in [
-                        dtype.rich_text,
-                        dtype.short_text,
-                        dtype.array,
-                        dtype.tsarray,
-                        dtype.video,
-                        dtype.audio,
-                        dtype.image,
-                    ]
-                    else 1
-                    for x in type_information.dtypes.values()
-                ]
-            )
-            * 200
-        )
+    if problem_definition.time_aim is None:
+        # 5 days
+        problem_definition.time_aim = 3 * 24 * 3600
 
-    if problem_definition.time_aim is not None:
+    # Encoders are assigned 1/3 of the time unless a user overrides this (equal time per encoder)
+    if problem_definition.seconds_per_encoder is None:
         nr_trainable_encoders = len(
             [
                 x
                 for x in features.values()
-                if eval(x.encoder["module"].split(".")[1]).is_trainable_encoder
+                if eval(x.encoder["module"]).is_trainable_encoder
             ]
         )
-        nr_mixers = len(list(outputs.values())[0].mixers)
-        encoder_time_budget_pct = max(
-            3.3 / 5, 1.5 + np.log(nr_trainable_encoders + 1) / 5
-        )
+        if nr_trainable_encoders > 0:
+            problem_definition.seconds_per_encoder = 0.33 * problem_definition.time_aim / nr_trainable_encoders
 
-        if nr_trainable_encoders == 0:
-            problem_definition.seconds_per_encoder = 0
+    # Mixers are assigned 1/3 of the time aim (or 2/3 if there are no trainable encoders )\
+    # unless a user overrides this (equal time per mixer)
+    if problem_definition.seconds_per_mixer is None:
+        nr_mixers = len(list(outputs.values())[0].mixers)
+        if problem_definition.seconds_per_encoder is None:
+            problem_definition.seconds_per_mixer = 0.66 * problem_definition.time_aim / nr_mixers
         else:
-            problem_definition.seconds_per_encoder = int(
-                problem_definition.time_aim
-                * (encoder_time_budget_pct / nr_trainable_encoders)
-            )
-        problem_definition.seconds_per_mixer = int(
-            problem_definition.time_aim * ((1 / encoder_time_budget_pct) / nr_mixers)
-        )
+            problem_definition.seconds_per_mixer = 0.33 * problem_definition.time_aim / nr_mixers
 
     return JsonAI(
         cleaner=None,
@@ -533,6 +515,10 @@ def _add_implicit_values(json_ai: JsonAI) -> JsonAI:
             mixers[i]["args"]["input_cols"] = mixers[i]["args"].get(
                 "input_cols", "$input_cols"
             )
+            mixers[i]["args"]["target_encoder"] = mixers[i]["args"].get(
+                "target_encoder", "$encoders[self.target]"
+            )
+            mixers[i]["args"]["use_optuna"] = True
         elif mixers[i]["module"] == "Regression":
             mixers[i]["args"]["target"] = mixers[i]["args"].get("target", "$target")
             mixers[i]["args"]["dtype_dict"] = mixers[i]["args"].get(
@@ -549,6 +535,9 @@ def _add_implicit_values(json_ai: JsonAI) -> JsonAI:
             mixers[i]["args"]["input_cols"] = mixers[i]["args"].get(
                 "input_cols", "$input_cols"
             )
+            mixers[i]["args"]["target_encoder"] = mixers[i]["args"].get(
+                "target_encoder", "$encoders[self.target]"
+            )
         elif mixers[i]["module"] == "SkTime":
             mixers[i]["args"]["target"] = mixers[i]["args"].get("target", "$target")
             mixers[i]["args"]["dtype_dict"] = mixers[i]["args"].get(
@@ -557,6 +546,8 @@ def _add_implicit_values(json_ai: JsonAI) -> JsonAI:
             mixers[i]["args"]["ts_analysis"] = mixers[i]["args"].get(
                 "ts_analysis", "$ts_analysis"
             )
+            # enforce fit_on_all if this mixer is specified
+            problem_definition.fit_on_all = True
 
     ensemble = json_ai.outputs[json_ai.problem_definition.target].ensemble
     ensemble["args"]["target"] = ensemble["args"].get("target", "$target")
@@ -566,10 +557,6 @@ def _add_implicit_values(json_ai: JsonAI) -> JsonAI:
     for name in json_ai.features:
         if json_ai.features[name].dependency is None:
             json_ai.features[name].dependency = []
-        if json_ai.features[name].data_dtype is None:
-            json_ai.features[name].data_dtype = (
-                json_ai.features[name].encoder["module"].split(".")[0].lower()
-            )
 
     # Add "hidden" fields
     hidden_fields = {
@@ -603,7 +590,7 @@ def _add_implicit_values(json_ai: JsonAI) -> JsonAI:
             "module": "model_analyzer",
             "args": {
                 "stats_info": "$statistical_analysis",
-                "ts_cfg": "$problem_definition.timeseries_settings",
+                "tss": "$problem_definition.timeseries_settings",
                 "accuracy_functions": "$accuracy_functions",
                 "predictor": "$ensemble",
                 "data": "encoded_test_data",
@@ -723,8 +710,9 @@ def code_from_json_ai(json_ai: JsonAI) -> str:
             )
         )
         dependency_dict[col_name] = []
-        dtype_dict[col_name] = f"""'{list(json_ai.outputs.values())[0].data_dtype}'"""
-        json_ai.features[col_name] = Feature(encoder=encoder_dict[col_name])
+        data_dtype = list(json_ai.outputs.values())[0].data_dtype
+        dtype_dict[col_name] = f"""'{data_dtype}'"""
+        json_ai.features[col_name] = Feature(encoder=encoder_dict[col_name], data_dtype=data_dtype)
 
     # ----------------- #
 
@@ -971,8 +959,8 @@ self.mode = 'train'
 # Extract data
 # --------------- #
 # Extract the featurized data
-encoded_old_data = new_data['old']
-encoded_new_data = new_data['new']
+encoded_old_data = old_data if old_data is not None else pd.DataFrame()
+encoded_new_data = new_data
 
 # --------------- #
 # Adjust (Update) Mixers
@@ -989,7 +977,7 @@ for mixer in self.mixers:
     # Learn Body
     # ----------------- #
 
-    learn_body = f"""
+    learn_body = """
 self.mode = 'train'
 
 # Perform stats analysis
@@ -1022,9 +1010,7 @@ self.analyze_ensemble(enc_train_test)
 if self.problem_definition.fit_on_all:
 
     log.info("Adjustment on validation requested.")
-    update_data = {{"new": enc_train_test["test"], "old": ConcatedEncodedDs([enc_train_test["train"], enc_train_test["dev"]])}}  # noqa
-
-    self.adjust(update_data)
+    self.adjust(enc_train_test["test"], ConcatedEncodedDs([enc_train_test["train"], enc_train_test["dev"]]))
 
 """
     learn_body = align(learn_body, 2)
@@ -1033,8 +1019,9 @@ if self.problem_definition.fit_on_all:
     # ----------------- #
 
     predict_body = f"""
-# Remove columns that user specifies to ignore
 self.mode = 'predict'
+
+# Remove columns that user specifies to ignore
 log.info(f'Dropping features: {{self.problem_definition.ignore_features}}')
 data = data.drop(columns=self.problem_definition.ignore_features, errors='ignore')
 for col in self.input_cols:
@@ -1122,7 +1109,7 @@ class Predictor(PredictorInterface):
         data = data.drop(columns=self.problem_definition.ignore_features, errors='ignore')
 {learn_body}
 
-    def adjust(self, new_data: Dict[str, pd.DataFrame]) -> None:
+    def adjust(self, new_data: pd.DataFrame, old_data: Optional[pd.DataFrame] = None) -> None:
         # Update mixers with new information
 {adjust_body}
 
