@@ -280,8 +280,12 @@ def _remove_columns(data: pd.DataFrame, identifiers: Dict[str, object], target:
     data = deepcopy(data)
     to_drop = [*[x for x in identifiers.keys() if x != target],
                *[x for x in data.columns if x in dtype_dict and dtype_dict[x] == dtype.invalid]]
+
     exceptions = ["__mdb_make_predictions"]
-    to_drop = [x for x in to_drop if x in data.columns]
+    if timeseries_settings.group_by is not None:
+        exceptions += timeseries_settings.group_by
+
+    to_drop = [x for x in to_drop if x in data.columns and x not in exceptions]
     data = data.drop(columns=to_drop)
 
     if mode == "train":
@@ -289,7 +293,7 @@ def _remove_columns(data: pd.DataFrame, identifiers: Dict[str, object], target:
     if mode == "predict":
         if (
             target in data.columns
-            and not timeseries_settings.use_previous_target
+            and (not timeseries_settings.is_timeseries or not timeseries_settings.use_previous_target)
             and not anomaly_detection
         ):
             data = data.drop(columns=[target])
@@ -298,7 +302,6 @@ def _remove_columns(data: pd.DataFrame, identifiers: Dict[str, object], target:
     for name in list(data.columns):
         if name not in dtype_dict and name not in exceptions:
             data = data.drop(columns=[name])
-
     return data
 
 
