@@ -40,6 +40,13 @@ def transform_timeseries(
     ob_arr = tss.order_by
     window = tss.window
 
+    if tss.use_previous_target and target not in data.columns:
+        raise Exception(f"Cannot transform. Missing historical values for target column {target} (`use_previous_target` is set to True).")  # noqa
+
+    for hcol in tss.historical_columns:
+        if hcol not in data.columns or data[hcol].isna().any():
+            raise Exception(f"Cannot transform. Missing values in historical column {hcol}.")
+
     if '__mdb_make_predictions' in original_df.columns:
         index = original_df[original_df['__mdb_make_predictions'].map(
             {'True': True, 'False': False, True: True, False: False}).isin([True])]
@@ -155,14 +162,14 @@ def transform_timeseries(
         if tss.allow_incomplete_history:
             log.warning("Forecasting with incomplete historical context, predictions might be subpar")
         else:
-            raise Exception(f'Not enough historical context to make a timeseries prediction. Please provide a number of rows greater or equal to the window size. If you can\'t get enough rows, consider lowering your window size. If you want to force timeseries predictions lacking historical context please set the `allow_incomplete_history` timeseries setting to `True`, but this might lead to subpar predictions.') # noqa
+            raise Exception(f'Not enough historical context to make a timeseries prediction. Please provide a number of rows greater or equal to the window size - currently (number_rows, window_size) = ({min(group_lengths)}, {tss.window}). If you can\'t get enough rows, consider lowering your window size. If you want to force timeseries predictions lacking historical context please set the `allow_incomplete_history` timeseries setting to `True`, but this might lead to subpar predictions depending on the mixer.') # noqa
 
     df_gb_map = None
     if n_groups > 1:
         df_gb_list = list(combined_df.groupby(tss.group_by))
         df_gb_map = {}
         for gb, df in df_gb_list:
-            df_gb_map['_' + '_'.join(gb)] = df
+            df_gb_map['_' + '_'.join(str(gb))] = df
 
     timeseries_row_mapping = {}
     idx = 0
@@ -190,7 +197,6 @@ def transform_timeseries(
 
     del combined_df['original_index']
 
-    # return combined_df, secondary_type_dict, timeseries_row_mapping, df_gb_map
     return combined_df
 
 
@@ -336,11 +342,4 @@ def _ts_add_future_target(df, target, nr_predictions, data_dtype, mode):
         df[col_name] = next_target_value_arr
         df[col_name] = df[col_name].fillna(value=np.nan)
 
-    # drop rows with incomplete target info.
-    if mode == 'train':
-        for col in [f'{target}_timestep_{i}' for i in range(1, nr_predictions)]:
-            if '__mdb_make_predictions' not in df.columns:
-                df['__mdb_make_predictions'] = True
-            df.loc[df[col].isna(), ['__mdb_make_predictions']] = False
-
     return df
